// PrefsDialog.cc - Preferences dialog class for QTM
//
// Written by Matthew J Smith, 3rd July 2006
//
// This is released under the GNU General Public License as is the rest of
// the QTM package
// http://catkin.blogistan.co.uk

#include "PrefsDialog.h"

#include <QWidget>
#include <QDialog>
#include <QFileDialog>
#include <QString>
#include <QHostInfo>
#include <QMessageBox>
#include <QWhatsThis>

PrefsDialog::PrefsDialog( QWidget *parent )
  : QDialog( parent )
{
  setupUi( this );
  nvh = false;

  cbBlogType->setCurrentIndex( cbBlogType->count()-1 );

#ifdef Q_WS_MAC
#if QT_VERSION >= 0x040200
  // The STI double/single click functionality does not work on a Mac,
  // so these widgets are made invisible
  label_10->setVisible( false );
  cbSTI2ClickFunction->setVisible( false );
#endif
#endif
}

void PrefsDialog::on_pbBrowse_clicked()
{
  QString dir = 
    QFileDialog::getExistingDirectory( 0,
				       tr( "Choose a directory" ),
				       "/home" );

  if( !dir.isEmpty() )
    leLocalDir->setText( dir );		             
}

void PrefsDialog::on_pbForget_clicked()
{
  leServer->clear();
  leLocation->clear();
  leLogin->clear();
  lePort->clear();
  lePassword->clear();
}

void PrefsDialog::on_okButton_clicked() // slot
{
  QHostInfo::lookupHost( leServer->text(), this, SLOT( handleHostInfo( QHostInfo ) ) );
}

void PrefsDialog::handleHostInfo( const QHostInfo &hostInfo )
{
  if( hostInfo.error() == QHostInfo::NoError ) {
    nvh = false;
    accept();
  }
  else {
    if( QMessageBox::question( this, tr( "Cannot find host" ),
			   tr( "Cannot find the host you specified:\n%1\n\n"
			       "Go back and correct it?" )
			   .arg( leServer->text() ),
			   tr( "&Yes" ), tr( "&No" ),
			       QString(), 0, 1 ) ) {
      nvh = true;
      accept();
    }
  }
}

void PrefsDialog::on_pbWhatsThis_clicked() // slot
{
  QWhatsThis::enterWhatsThisMode();
}

void PrefsDialog::on_cbBlogType_currentIndexChanged( int newIndex )
{
  switch( newIndex ) {
  case 0: // wordpress.com
    leServer->setText( "yourblog.wordpress.com" );
    leLocation->setText( "/xmlrpc.php" );
    break;
  case 1: // TypePad
    leServer->setText( "www.typepad.com" );
    leLocation->setText( "/t/api" );
    break;
  case 2: // SquareSpace
    leServer->setText( "www.squarespace.com" );
    leLocation->setText( "/do/process/external/PostInterceptor" );
    break;
  case 3: // self-hosted
    leServer->clear();
    leLocation->clear();
    break;
  }
}

bool PrefsDialog::noValidHost()
{
  return nvh;
}
