// QuickpostTemplateDialog.cc - Quickpost template configuration dialog for QTM
//
// Written by Matthew J Smith, 13th Feb 2007
//
// This is released under the GNU General Public License as is the rest of
// the QTM package
// http://qtm.blogistan.co.uk

#include "QuickpostTemplateDialog.h"
#include "ui_QuickpostTemplateForm.h"

#include <QWidget>
#include <QDialog>
#include <QListWidget>
#include <QListWidgetItem>
#include <QComboBox>
#include <QStringList>
#include <QList>
#include <QWhatsThis>
#include <QMessageBox>

QuickpostTemplateDialog::QuickpostTemplateDialog( QStringList &titles,
						  QStringList &templates,
						  QList<int> &pubStates,
						  QWidget *parent )
  : QDialog( parent )
{
  setupUi( this );

  teTemplateText->setWhatsThis( tr( "<p>Enter the text of your template here, in HTML "
				    "format.</p>"
				    "<p>Two place-markers are supported:</p>"
				    "<p><strong>%url%</strong> - The web location (URL) of "
				    "the page being blogged, and<br />"
				    "<strong>%title%</strong> - the page's title.</p>" ) );
  cbDefaultPublishStatus->setWhatsThis( tr( "<p>Set this to Draft or Publish if you always want "
					    "posts based on this template to have that status.</p>"
					    "<p>Otherwise, the post will have the status you specify in "
					    "the Preferences window.</p>" ) );

  _templateTitles = titles;
  _templateStrings = templates;
  _defaultPublishStates = pubStates;
  originalTitles = titles;
  originalStrings = templates;
  originalPublishStates = pubStates;

  // lowestNumber = lwTemplates->count();

  hboxLayout->setStretchFactor( gridLayout, 2 );
  hboxLayout->setStretchFactor( gridLayout1, 3 );

  /*titlesModel = new QStringListModel( _templateTitles, this );
  lwTemplates->setModel( titlesModel );
  */
  lwTemplates->addItems( _templateTitles );

  doingNewTemplate = false;

  addTemplate = new QAction( tr( "&Add template" ), lwTemplates );
  removeTemplate = new QAction( tr( "&Remove this template" ), lwTemplates );
  lwTemplates->addAction( addTemplate );
  lwTemplates->addAction( removeTemplate );
  lwTemplates->setContextMenuPolicy( Qt::ActionsContextMenu );

  connect( addTemplate, SIGNAL( triggered( bool ) ),
	   this, SLOT( doNewTemplate() ) );
  connect( removeTemplate, SIGNAL( triggered( bool ) ),
	   this, SLOT( removeThisTemplate() ) );
  connect( lwTemplates, SIGNAL( currentRowChanged( int ) ),
	   this, SLOT( changeListIndex( int ) ) );
}

void QuickpostTemplateDialog::changeListIndex( int index )
{
  doingNewTemplate = false;

  currentRow = index;
  // lwTemplates->setCurrentIndex( index );
  currentTemplateTitle = _templateTitles[currentRow];
  currentTemplateString = _templateStrings[currentRow];

  leName->setText( _templateTitles[currentRow] );
  teTemplateText->setPlainText( _templateStrings[currentRow] );
  cbDefaultPublishStatus->setCurrentIndex( _defaultPublishStates[currentRow] );
  leName->setEnabled( true );
  teTemplateText->setEnabled( true );
  cbDefaultPublishStatus->setEnabled( true );
}

void QuickpostTemplateDialog::doNewTemplate()
{
  leName->setEnabled( true );
  teTemplateText->setEnabled( true );
  cbDefaultPublishStatus->setCurrentIndex( 2 );
  if( cbDefaultPublishStatus->count() == 4 ) // if it's been disabled and blank
    cbDefaultPublishStatus->removeItem( 3 ); // remove the blank item
  cbDefaultPublishStatus->setEnabled( true );
  pbAccept->setEnabled( true );

  doingNewTemplate = true;
  leName->clear();
  teTemplateText->clear();
  leName->setFocus( Qt::OtherFocusReason );
}

void QuickpostTemplateDialog::removeThisTemplate()
{
  int c = lwTemplates->currentRow();
  // lwTemplates->takeItem( c );

  if( lwTemplates->count() == 1 ) {
    lwTemplates->disconnect( SIGNAL( currentRowChanged( int ) ) );
    lwTemplates->clear();
    _templateTitles.clear();
    _templateStrings.clear();
    _defaultPublishStates.clear();
    leName->clear();
    leName->setEnabled( false );
    teTemplateText->clear();
    teTemplateText->setEnabled( false );
    cbDefaultPublishStatus->addItem( "" );
    cbDefaultPublishStatus->setCurrentIndex( 3 );
    cbDefaultPublishStatus->setEnabled( false );
    pbAccept->setEnabled( false );
  } else {
    lwTemplates->takeItem( c );
    _templateTitles.removeAt( c );
    _templateStrings.removeAt( c );
    _defaultPublishStates.removeAt( c );
  }
  qDebug( "exiting" );
  /*if( originalTitles.size() < lowestNumber )
    lowestNumber = titles.size(); */
}

void QuickpostTemplateDialog::on_pbAccept_clicked()
{
  if( doingNewTemplate ) {
    _templateTitles.append( leName->text() );
    _templateStrings.append( teTemplateText->toPlainText() );
    _defaultPublishStates.append( cbDefaultPublishStatus->currentIndex() );
    if( lwTemplates->count() == 1 )
      connect( lwTemplates, SIGNAL( currentRowChanged( int ) ),
	       this, SLOT( changeListIndex( int ) ) );
    QListWidgetItem *newItem = new QListWidgetItem( leName->text() );
    lwTemplates->addItem( newItem );
    lwTemplates->setCurrentItem( newItem );
  } else {
    lwTemplates->item( currentRow )->setText( leName->text() );
    _templateTitles[currentRow] = leName->text();
    _templateStrings[currentRow] = teTemplateText->toPlainText();
    _defaultPublishStates[currentRow] = cbDefaultPublishStatus->currentIndex();
  }

  doingNewTemplate = false;
}

void QuickpostTemplateDialog::on_pbWhatsThis_clicked()
{
  QWhatsThis::enterWhatsThisMode();
}

void QuickpostTemplateDialog::on_pbReset_clicked()
{
  if( QMessageBox::question( 0, tr( "QTM" ),
			     tr( "Are you sure you wish to lose all your changes?" ), 
			     QMessageBox::Yes | QMessageBox::No, QMessageBox::No )
      == QMessageBox::Yes ) {
    lwTemplates->disconnect( SIGNAL( currentRowChanged( int ) ) );
    lwTemplates->clear();
    //qDebug( "cleared templates list widget" );
    leName->clear();
    teTemplateText->clear();

    _templateTitles = originalTitles;
    _templateStrings = originalStrings;
    _defaultPublishStates = originalPublishStates;
    //qDebug( "rewritten all lists" );

    lwTemplates->addItems( _templateTitles );
    //qDebug( "added titles to list " );
    leName->clear();
    teTemplateText->clear();
    leName->setEnabled( false );
    teTemplateText->setEnabled( false );
    doingNewTemplate = false;
    connect( lwTemplates, SIGNAL( currentRowChanged( int ) ),
	     this, SLOT( changeListIndex( int ) ) );
  }
}

void QuickpostTemplateDialog::on_buttonBox_accepted()
{
  on_pbAccept_clicked();
  accept();
}
