// SafeHttp.cc - Safe wrapper to QHttp which checks for the existence of
// a given host and location before attempting to post data to it (to
// prevent crashes).
//
// Copyright 2006 Matthew J Smith
// Distributed under the GNU General Public License, version 2

#include "SafeHttp.h"

#include <QtNetwork>

SafeHttp::SafeHttp( QObject *parent )
  : QHttp( parent )
{

}

SafeHttp::SafeHttp( const QString &hostName,
		    quint16 port, QObject *parent )
  : QHttp( parent )
{
  httpHost = hostName;
}

SafeHttp::~SafeHttp()
{
}

void SafeHttp::safePost( const QString &host, 
			 const QString &path,
			 const QByteArray &data,
			 QIODevice *dataTo )
{
  httpHost = host;
  location = path;
  httpData = data;
  httpDataTo = dataTo;

  QHostInfo::lookupHost( host, this,
			 SLOT( handleLookupResult( QHostInfo ) ) );
}

void SafeHttp::handleLookupResult( const QHostInfo &hostInfo )
{
  if( hostInfo.error() != QHostInfo::NoError )
    emit hostLookupFailed();
  else {
    setHost( httpHost );
    post( location, httpData, httpDataTo );
  }
}

void SafeHttp::setCurrentHttpBusiness( int chb )
{
  _currentHttpBusiness = chb;
}

int SafeHttp::currentHttpBusiness()
{
  return _currentHttpBusiness;
}
