/*******************************************************************************
 *
 *  SuperMenu.cc - Default menubar for QTM on the Mac
 *
 *  Copyright (C) Matthew J Smith, 6th May 2007
 *  Distributed under the GNU General Public Licence, along with
 *  the rest of QTM.
 *
 *******************************************************************************/

#include <QMenu>
#include <QAction>
#include <QMenuBar>
#include <QSettings>
#include <QString>
#include <QStringList>
#include <QMessageBox>
#include <QFileDialog>
#include <QKeySequence>

#include "SuperMenu.h"
#include "catkin.h"
#include "SysTrayIcon.h"

SuperMenu::SuperMenu( QWidget *parent, SysTrayIcon *sti )
  : QMenuBar( parent )
{
  fileMenu = addMenu( tr( "&File" ) );
  newEntryAction = fileMenu->addAction( tr( "&New entry" ), this, SLOT( newEntry() ) );
  newEntryAction->setShortcut( QKeySequence( tr( "Ctrl+N" ) ) );
  openAction = fileMenu->addAction( tr( "&Open ..." ), this, SLOT( choose() ) );
  openAction->setShortcut( QKeySequence( tr( "Ctrl+O" ) ) );
  quitAction = fileMenu->addAction( tr( "&Quit" ), this, SLOT( quit() ) );
  quitAction->setMenuRole( QAction::QuitRole );
  _sti = sti;
}

void SuperMenu::newEntry()
{
  Catkin *c = new Catkin;
  c->setSTI( _sti );
  c->setWindowTitle( QObject::tr( "QTM - new entry [*]" ) );
  c->show();
#if defined QT_VERSION >= 0x040300
  QApplication::alert( c );
#endif
}

void SuperMenu::choose()
{
  QSettings settings;
  QString localStorageFileExtn, localStorageDirectory;
  QStringList filesSelected;

  settings.beginGroup( "account" );
  localStorageFileExtn = settings.value( "localStorageFileExtn", "cqt" ).toString();
  localStorageDirectory = settings.value( "localStorageDirectory", "" ).toString();
  settings.endGroup();

  QString extn( QString( "%1 (*.%2)" ).arg( tr( "Blog entries" ) )
		.arg( localStorageFileExtn ) );
  
  QString fn =
    QFileDialog::getOpenFileName( 0, tr( "Choose a file to open" ),
    localStorageDirectory, extn );

  if( !fn.isEmpty() ) {
    Catkin *e = new Catkin( true );
    if( !e->load( fn, true ) ) {
      QMessageBox::warning( 0, "QTM",
			    tr( "Could not load the file you specified." ),
			    QMessageBox::Cancel, QMessageBox::NoButton );
      e->deleteLater();
    } else {
      e->setSTI( _sti );
      e->show();
      e->activateWindow();
#if defined QT_VERSION >= 0x040300
      QApplication::alert( e );
#endif
    }
  }
}


void SuperMenu::quit()
{
  QCoreApplication::quit();
}
