/*********************************************************************************

    QTM - Qt-based blog manager
    Copyright (C) 2006 Matthew J Smih

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License (version 2), as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/

// SysTrayIcon.cc - System tray icon for QTM

#include <QtGlobal>
//#include "useSTI.h"

#if QT_VERSION >= 0x040200
#ifdef USE_SYSTRAYICON

#include <QSystemTrayIcon>
#include <QMenu>
#include <QAction>
#include <QSettings>
#include <QString>
#include <QStringList>
#include <QFileDialog>
#include <QMessageBox>
#include <QApplication>
#include <QCoreApplication>
#include <QClipboard>
#include <QUrl>
#include <QHttpRequestHeader>
#include <QList>
#include <QFile>
#include <QTextStream>

#include <QtXml>
#include <cstdio>

#include "SafeHttp.h"
#include "SysTrayIcon.h"
#include "catkin.h"
#include "QuickpostTemplate.h"
#include "QuickpostTemplateDialog.h"

//#include "ui_QuickpostTemplateForm.h"

#ifdef Q_WS_X11
#include "susesystray.xpm"
#else
#include "winsystray.xpm"
#endif

SysTrayIcon::SysTrayIcon( QObject *parent )
  : QSystemTrayIcon( parent )
{
  bool newWindow;

  QCoreApplication::setOrganizationName( "Catkin Project" );
  QCoreApplication::setOrganizationDomain( "catkin.blogistan.co.uk" );
  QCoreApplication::setApplicationName( "QTM" );

  QSettings settings;
  settings.beginGroup( "sysTrayIcon" );
  newWindow = settings.value( "newWindowAtStartup", true ).toBool();
  doubleClickFunction = settings.value( "doubleClickFunction", 0 ).toInt();
  settings.endGroup();

#ifndef Q_WS_MAC
  switch( doubleClickFunction ) {
  case 0:
    setToolTip( tr( "QTM - Double click for \nnew entry" ) ); break;
  case 1:
    setToolTip( tr( "QTM - Double click for \nQuick Post" ) ); break;
  }
#endif

  setIcon( QIcon( QPixmap( sysTrayIcon_xpm ) ) );

  newWindowAtStartup = new QAction( tr( "New entry at startup" ), this );
  newWindowAtStartup->setCheckable( true );
  newWindowAtStartup->setChecked( newWindow );
  connect( newWindowAtStartup, SIGNAL( toggled( bool ) ),
	   this, SLOT( setNewWindowAtStartup( bool ) ) );

  menu = new QMenu;
  menu->addAction( tr( "New entry" ), this, SLOT( newDoc() ) );
  menu->addAction( tr( "Open ..." ), this, SLOT( choose() ) );
  menu->addAction( tr( "Quick post" ), this, SLOT( quickpost() ) );
  abortAction = menu->addAction( tr( "Abort quick post" ), this, SLOT( abortQP() ) );
  abortAction->setEnabled( false );
  templateMenu = menu->addMenu( tr( "Quickpost templates" ) );
  templateMenu->setObjectName( "templateMenu" );
  configureTemplates = new QAction( tr( "&Configure ..." ), 0 );
  connect( configureTemplates, SIGNAL( triggered( bool ) ),
	   this, SLOT( configureQuickpostTemplates() ) );
  templateMenu->addAction( configureTemplates );
  // configureTemplates->setMenu( templateMenu );
  configureTemplates->setObjectName( "QTM Configure Templates" );
  setupQuickpostTemplates();
  menu->addAction( newWindowAtStartup );
#ifndef Q_WS_MAC
  menu->addSeparator();
#endif
#ifndef SUPERMENU
  quitAction = menu->addAction( tr( "Quit" ), this, SLOT( doQuit() ) );
#endif
  // quitAction->setMenuRole( QAction::QuitRole );
  menu->setObjectName( "iconMenu" );
  setContextMenu( menu );
#ifdef Q_WS_MAC
  qt_mac_set_dock_menu( menu );
#endif
  //show();

#ifndef Q_WS_MAC
  connect( this, SIGNAL( activated( QSystemTrayIcon::ActivationReason ) ),
	   this, SLOT( iconActivated( QSystemTrayIcon::ActivationReason ) ) );
#endif
  /*connect( http, SIGNAL( done( bool ) ),
    this, SLOT( handleDone( bool ) ) ); */

  if( newWindow ) {
    Catkin *c = new Catkin;
    c->setSTI( this );
    c->setWindowTitle( tr( "QTM - new entry [*]" ) );
    c->show();
    c->activateWindow();
  }
  //cbtextIsURL = Untested;
  http = new SafeHttp;
  httpBusy = false;
  templateQPActive = false;
  activeTemplate = -1;
}

SysTrayIcon::~SysTrayIcon()
{
}

void SysTrayIcon::setDoubleClickFunction( int func )
{
  doubleClickFunction = func;

#ifndef Q_WS_MAC
  switch( doubleClickFunction ) {
  case 0:
    setToolTip( tr( "QTM - Double click for \nnew entry" ) ); break;
  case 1:
    setToolTip( tr( "QTM - Double click for \nQuick Post" ) ); break;
  }
#endif
}

void SysTrayIcon::newDoc()
{
  Catkin *c = new Catkin;
  c->setSTI( this );
  c->show();
  c->activateWindow();
#if QT_VERSION >= 0x040300
  QApplication::alert( c );
#endif
}

void SysTrayIcon::choose()
{
  QSettings settings;
  QString localStorageFileExtn, localStorageDirectory;
  QStringList filesSelected;

  settings.beginGroup( "account" );
  localStorageFileExtn = settings.value( "localStorageFileExtn", "cqt" ).toString();
  localStorageDirectory = settings.value( "localStorageDirectory", "" ).toString();
  settings.endGroup();

  QString extn( QString( "%1 (*.%2)" ).arg( tr( "Blog entries" ) )
		.arg( localStorageFileExtn ) );
  /*  QFileDialog *fd = new QFileDialog;
  fd->setFileMode( QFileDialog::AnyFile );
  fd->setAcceptMode( QFileDialog::AcceptOpen );
  fd->setWindowTitle( tr( "Choose a file to open" ) );
  fd->setFilter( extn );
  if( fd->exec() ) {
    filesSelected = fd->selectedFiles();
    fn = filesSelected[0];*/
  
  QString fn =
    QFileDialog::getOpenFileName( 0, tr( "Choose a file to open" ),
    localStorageDirectory, extn );

  if( !fn.isEmpty() ) {
    Catkin *e = new Catkin( true );
    if( !e->load( fn, true ) ) {
      /*      e->show();
	      else { */
#ifdef Q_WS_MAC
      QMessageBox::warning( 0, "QTM",
			    tr( "Could not load the file you specified." ),
			    QMessageBox::Cancel, QMessageBox::NoButton );
#else
      showMessage( tr( "Error" ), 
		   tr( "Could not load the file you specified." ),
		   QSystemTrayIcon::Warning );
#endif
      e->deleteLater();
    } else {
      e->setSTI( this );
      e->show();
      e->activateWindow();
#if QT_VERSION >= 0x040300
      QApplication::alert( e );
#endif
    }
  }
}

void SysTrayIcon::iconActivated( QSystemTrayIcon::ActivationReason ar )
{
#ifndef Q_WS_MAC
  if( ar == QSystemTrayIcon::DoubleClick )
    switch( doubleClickFunction ) {
    case 0: newDoc(); break;
    case 1: quickpost(); break;
    }

#ifdef Q_WS_X11
  if( ar == QSystemTrayIcon::MiddleClick )
    quickpost( QClipboard::Selection );
#endif
#endif
}

void SysTrayIcon::quickpost( QClipboard::Mode mode )
{
  cbtext = QApplication::clipboard()->text( mode );

  if( cbtext == "" )
    showMessage( tr( "No selection!" ),
		 tr( "No web location specified to blog about." ),
		 QSystemTrayIcon::Warning );
  else {
    if( !cbtext.startsWith( "http://" ) ) { 
      // If it's not obviously an URL.
      doQP( "" );
    }
    else {
      if( !QUrl( cbtext, QUrl::StrictMode ).isValid() ) {
	if( cbtext.contains( QChar( ' ' ) ) ) {
	  doQP( "" );
	}
	else
	  showMessage( tr( "Invalid web location" ),
		       tr( "Quick Post requires valid web location." ),
		       QSystemTrayIcon::Warning );
      }
      else {
      // Otherwise, it's an URL, and has to be downloaded to extract the title.
	if( !httpBusy ) {
	  QString withoutHttp = cbtext.section( "http://", 1 );
	  QString host = withoutHttp.section( "/", 0, 0 );
	  QString loc = withoutHttp.section( '/', 1, -1, QString::SectionIncludeLeadingSep );
	  http->setHost( host );
	  http->get( loc );
	  abortAction->setEnabled( true );
	  httpBusy = true;
	  templateQPActive = false;
	  connect( http, SIGNAL( done( bool ) ),
		   this, SLOT( handleDone( bool ) ) );
	  connect( http, SIGNAL( readyRead( const QHttpResponseHeader & ) ),
		   this, SLOT( handleResponseHeader( const QHttpResponseHeader & ) ) );
	  connect( http, SIGNAL( hostLookupFailed() ),
		   this, SLOT( handleHostLookupFailed() ) );
	}
      }
    }
  }
}


void SysTrayIcon::quickpostFromTemplate( int id, QString templateString )
{
  activeTemplate = id;
  cbtext = QApplication::clipboard()->text( QClipboard::Clipboard );

  if( !QUrl( cbtext, QUrl::StrictMode ).isValid() ) {
    showMessage( tr( "No selection!" ),
		 tr( "The selection is not a web location, or is invalid." ),
		 QSystemTrayIcon::Warning );
  }
  else {
    if( !httpBusy ) {
      QString withoutHttp = cbtext.section( "http://", 1 );
      QString host = withoutHttp.section( "/", 0, 0 );
      QString loc = withoutHttp.section( '/', 1, -1, QString::SectionIncludeLeadingSep );
      http->setHost( host );
      http->get( loc );
      abortAction->setEnabled( true );
      httpBusy = true;
      templateQPActive = true;
      connect( http, SIGNAL( done( bool ) ),
	       this, SLOT( handleDone( bool ) ) );
      connect( http, SIGNAL( readyRead( const QHttpResponseHeader & ) ),
	       this, SLOT( handleResponseHeader( const QHttpResponseHeader & ) ) );
      connect( http, SIGNAL( hostLookupFailed() ),
	       this, SLOT( handleHostLookupFailed() ) );
    }
    // cbtext = templateString.replace( "%url%", cbtext );
  }
}

void SysTrayIcon::setNewWindowAtStartup( bool nwas )
{
  QSettings settings;

  settings.beginGroup( "sysTrayIcon" );
  settings.setValue( "newWindowAtStartup", nwas );
  settings.endGroup();
}

// HTTP Quickpost handler routines

void SysTrayIcon::handleHostLookupFailed()
{
  disconnect( http );
  abortAction->setEnabled( false );
  httpBusy = false;
  http->abort();
  showMessage( tr( "Cannot find page" ),
	       tr( "QTM cannot find the resource you specified.\n"
		   "Check that it is spelled right and that your "
		   "network is working." ),
	       QSystemTrayIcon::Warning );
}

void SysTrayIcon::handleDone( bool error )
{
  QString ddoc, newTitle, errorString;
  QHttp::Error httpError;

  abortAction->setEnabled( false );
  httpBusy = false;

  if( error ) {
    httpError = http->error();
    switch( httpError ) {
    case QHttp::HostNotFound:
      errorString = tr( "Could not find the host." ); break;
    case QHttp::ConnectionRefused:
      errorString = tr( "Connection was refused." ); break;
    case QHttp::UnexpectedClose:
      errorString = tr( "Connection closed unexpectedly." ); break;
    case QHttp::InvalidResponseHeader:
    case QHttp::WrongContentLength:
    case QHttp::UnknownError:
      errorString = tr( "Document was not received correctly." ); break;
    }
    if( httpError != QHttp::Aborted )
      showMessage( tr( "Error" ), errorString, QSystemTrayIcon::Warning );
  }
  else {
    doQP( QString( responseData ) );
    responseData = "";
  }

  // http->close();
  http->disconnect();
}

void SysTrayIcon::handleResponseHeader( const QHttpResponseHeader &header ) // slot
{
  if( header.statusCode() == 404 ) {
    showMessage( tr( "Error 404" ),
		 tr( "The file or entry you specified could not be\n"
		     "found." ),
		 QSystemTrayIcon::Warning );
    http->disconnect();
    http->abort();
    abortAction->setEnabled( false );
    httpBusy = false;
    // http->deleteLater();
    // http = 0;
    // setNetworkActionsEnabled( false );
  }
  else
    responseData.append( http->readAll() );
}

void SysTrayIcon::doQuit()
{
  int catkins = 0;
  int a;

  QWidgetList tlw = qApp->topLevelWidgets();
  for( a = 0; a < tlw.size(); a++ ) {
    if( (QString( tlw[a]->metaObject()->className() ) == "Catkin")
	&& tlw[a]->isVisible() )
      catkins++;
  }

#ifndef NO_DEBUG_OUTPUT
  qDebug() << catkins << " main windows";
#endif
  
  //if( QApplication::topLevelWidgets().size() <= noWidgets )
  if( !catkins )
    QCoreApplication::quit();
  else {
    qApp->setQuitOnLastWindowClosed( true );
    qApp->closeAllWindows();
  }

}
//QCoreApplication::quit(); */

/*
#ifdef Q_WS_MAC
  if( QApplication::topLevelWidgets().isEmpty() ) {
    QCoreApplication::quit();
  } else {
    connect( qApp, SIGNAL( lastWindowClosed() ), qApp, SLOT( quit() ) );
    qApp->closeAllWindows();
  }
#else
  connect( qApp, SIGNAL( lastWindowClosed() ), qApp, SLOT( quit() ) );
  qApp->closeAllWindows();
#endif
} */

void SysTrayIcon::doQP( QString receivedText )
{
  QString newPost, newTitle;

  if( activeTemplate >= 0 ) {
    if( receivedText == "" )
      newPost = cbtext;
    else {
      newPost = templateList[activeTemplate];
      newPost.replace( "%url%", cbtext );
      if( receivedText.contains( "<title>", Qt::CaseInsensitive )
	  && receivedText.section( QRegExp( "<title>", Qt::CaseInsensitive ), 1 )
	  .section( '\n', 0 ).contains( "</title>", Qt::CaseInsensitive ) ) {
	newTitle = receivedText.section( QRegExp( "<title>", Qt::CaseInsensitive ), 1 )
	  .section( QRegExp( "</title>", Qt::CaseInsensitive ), 0, 0 )
	  .simplified();
	newPost.replace( "%title%", newTitle );
      }
    }
    newPost.replace( "\\n", "\n" );
  } else {
    if( receivedText == "" )
      newPost = cbtext;
    else {
	//// qDebug( receivedText.left( 300 ).toAscii().constData() );
	// The title check will accept flaky 1990s HTML - this isn't a browser
      if( receivedText.contains( "<title>", Qt::CaseInsensitive )
	  && receivedText.section( QRegExp( "<title>", Qt::CaseInsensitive ), 1 )
	  .section( '\n', 0 ).contains( "</title>", Qt::CaseInsensitive ) ) {
	newTitle = receivedText.section( QRegExp( "<title>", Qt::CaseInsensitive ), 1 )
	  .section( QRegExp( "</title>", Qt::CaseInsensitive ), 0, 0 )
	  .simplified();
	if( !templateQPActive )
	  newPost = QString( "<a title = \"%1\" href=\"%2\">%1</a>\n\n" )
	    .arg( newTitle )
	    .arg( cbtext );
      }
      else // Post has no valid title
	newPost = QString( tr( "<a href=\"%1\">Insert link text here</a>" ) )
	  .arg( cbtext );
    }
  }

  Catkin *c = new Catkin( newPost );
  c->setSTI( this );

  if( activeTemplate >= 0 ) {
    switch( defaultPublishStatusList[activeTemplate] ) {
    case 0:
    case 1:
      c->setPublishStatus( defaultPublishStatusList[activeTemplate] );
    }
  }
  c->show();
  c->activateWindow();
#if QT_VERSION >= 0x040300
  QApplication::alert( c );
#endif

  activeTemplate = -1;
  templateQPActive = false;
}

void SysTrayIcon::abortQP()
{
  http->abort();
  http->disconnect();
  abortAction->setEnabled( false );
  httpBusy = false;
}

void SysTrayIcon::setupQuickpostTemplates()
{
  QString templateFile;
  QString errorString;
  QString currentTitle, currentTemplate;
  int currentDefaultPublishStatus;
  int errorLine, errorCol;
  QDomNodeList titles, templateStrings, defaultPublishStates;
  int numTemplates, t;
  QTextStream ts( stdout );
  bool useDefaultPublishStatus, ok;

  QSettings settings;
  settings.beginGroup( "account" );
  templateFile = QString( "%1/qptemplates.xml" ).
    arg( settings.value( "localStorageDirectory", "" ).toString() );
  settings.endGroup();

  if( QFile::exists( templateFile ) ) {
    QDomDocument domDoc( "quickpostTemplates" );
    QFile file( templateFile );
    if( !domDoc.setContent( &file, true, &errorString, &errorLine, &errorCol ) )
      QMessageBox::warning( 0, tr( "Failed to read templates" ),
			    QString( tr( "Error: %1\n"
					 "Line %2, col %3" ) )
			    .arg( errorString ).arg( errorLine )
			    .arg( errorCol ) );
    else {
      titles = domDoc.elementsByTagName( "title" );
      templateStrings = domDoc.elementsByTagName( "templateString" );
      defaultPublishStates = domDoc.elementsByTagName( "defaultPublishStatus" );

      if( titles.size() ) {
	quickpostTemplateActions.clear();
	numTemplates = ( titles.size() >= templateStrings.size() ?
			 titles.size() : templateStrings.size() );
	useDefaultPublishStatus = (defaultPublishStates.size() < numTemplates) ?
	  false : true;

	// qDebug( "Populating template menu" );
	templateMenu->addSeparator();
	templateTitleList.clear();
	templateList.clear();
	for( t = 0; t < numTemplates; t++ ) {
	  currentTitle = titles.item( t ).toElement().text();
	  currentTemplate = templateStrings.item( t ).toElement().text();
	  currentDefaultPublishStatus = useDefaultPublishStatus ? 
	    defaultPublishStates.item( t ).toElement().text().toInt( &ok ) : 2;
	  templateTitleList.append( currentTitle );
	  templateList.append( currentTemplate );
	  if( ok ) {
	    switch( currentDefaultPublishStatus ) {
	    case 0:
	    case 1:
	      defaultPublishStatusList.append( currentDefaultPublishStatus ); break;
	    default:
	      defaultPublishStatusList.append( 2 );
	    }
	  }
	  else
	    defaultPublishStatusList.append( 2 );
	  
	  ts << titles.item( t ).nodeValue();
	  ts << templateStrings.item( t ).nodeValue();
	  quickpostTemplateActions.append( new QuickpostTemplate( t, currentTitle,
								  currentTemplate, 
								  templateMenu ) );
	  templateMenu->addAction( quickpostTemplateActions[t] );
	  connect( quickpostTemplateActions[t], 
		   SIGNAL( quickpostRequested( int, QString ) ),
		   this, SLOT( quickpostFromTemplate( int, QString ) ) );
        }
      }
      /*      else
	      qDebug( "No templates found." );*/
    }
    file.close();
  }
}

void SysTrayIcon::configureQuickpostTemplates()
{
  QDomDocument templateDocument;
  QDomElement quickpostTemplates;
  QString templateFileName;
  QString templateFile;
  QSettings settings;

  /*  QList<QString> titles, templateStrings;
      QStringListModel *titlesModel;*/

  QuickpostTemplateDialog *templateDialog = 
    new QuickpostTemplateDialog( templateTitleList, templateList,
				 defaultPublishStatusList );

  configureTemplates->setEnabled( false );
  configureTemplates->setMenu( 0 );
  if( templateDialog->exec() ) {
    // Set templates menu back up
    templateMenu->clear();
    templateTitleList = templateDialog->templateTitles();
    templateList = templateDialog->templateStrings();
    defaultPublishStatusList = templateDialog->defaultPublishStates();
    quickpostTemplates = templateDocument.createElement( "QuickpostTemplates" );
    int numTemplates = (templateTitleList.size() <= templateList.size()) ?
      templateTitleList.size() : templateList.size();
    quickpostTemplateActions.clear();
    for( int i = 0; i < numTemplates; i++ ) {
      quickpostTemplates.appendChild( templateElement( templateDocument,
						       templateTitleList[i],
						       templateList[i],
						       defaultPublishStatusList[i] ) );
      quickpostTemplateActions.append( new QuickpostTemplate( i,
							      templateTitleList[i],
							      templateList[i],
							      templateMenu ) );
      templateMenu->addAction( quickpostTemplateActions[i] );
      connect( quickpostTemplateActions[i],
	       SIGNAL( quickpostRequested( int, QString ) ),
	       this, SLOT( quickpostFromTemplate( int, QString ) ) );
    }
    templateDocument.appendChild( quickpostTemplates );
    templateMenu->addSeparator();
    configureTemplates->setEnabled( true );
    templateMenu->addAction( configureTemplates );

    // Save template XML file
    templateDocument.insertBefore( templateDocument.createProcessingInstruction( "xml", "version=\"1.0\"" ),
				   templateDocument.firstChild() );
    settings.beginGroup( "account" );
    templateFileName = QString( "%1/qptemplates.xml" )
      .arg( settings.value( "localStorageDirectory", "" ).toString() );
    settings.endGroup();
    QFile *templateFile = new QFile( templateFileName );
    if( templateFile->open( QIODevice::WriteOnly ) ) {
      QTextStream templateFileStream( templateFile );
      templateDocument.save( templateFileStream, 4 );
      templateFile->close();
    }
    else
      showMessage( tr( "Error" ),
		   tr( "Could not write to templates file" ), Warning );
    templateFile->deleteLater();
  }
  else {
    configureTemplates->setEnabled( true );
  }
  templateDialog->deleteLater();
}

QDomElement SysTrayIcon::templateElement( QDomDocument &doc,
					  QString title, QString templateString,
					  int defaultPublishStatus )
{
  QDomElement returnElement = doc.createElement( "template" );
  QDomElement titleElement = doc.createElement( "title" );
  titleElement.appendChild( QDomText( doc.createTextNode( title ) ) );
  QDomElement templateStringElement = doc.createElement( "templateString" );
  templateStringElement.appendChild( QDomText( doc.createTextNode( templateString ) ) );
  QDomElement defaultPublishStatusElement( doc.createElement( "defaultPublishStatus" ) );
  defaultPublishStatusElement.appendChild( QDomText( doc.createTextNode( QString::number( defaultPublishStatus ) ) ) );
  returnElement.appendChild( titleElement );
  returnElement.appendChild( templateStringElement );
  returnElement.appendChild( defaultPublishStatusElement );

  return returnElement;
}
#endif
#endif
