/*******************************************************************************
 
    QTM - Qt-based blog manager
    Copyright (C) 2006 Matthew J Smith

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License (version 2), as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/

// XmlRpcHandler.cc - XML-RPC handling routines for QTM

#include <QString>
#include <QList>
#include <QtXml>

#include "XmlRpcHandler.h"

XmlRpcHandler::XmlRpcHandler( int x )
{
  reqType = x;
  methodResponseFinished = false;
}

XmlRpcHandler::XmlRpcHandler()
{
  methodResponseFinished = false;
}

void XmlRpcHandler::setProtocol( int x )
{
  reqType = x;
}

bool XmlRpcHandler::startElement( const QString &, const QString &,
				  const QString &qName,
				  const QXmlAttributes &attr )
{
  if( qName == "struct" )
    currentString = "";
  if( (qName == "string" ) && (currentString.isNull() ) )
    currentString = "";
  return true;
}

bool XmlRpcHandler::characters( const QString &str )
{
  if( str.startsWith( "http://", Qt::CaseInsensitive ) )
    currentString += str.section( "//", 1 );
  else
    currentString += str;
  return true;
}

bool XmlRpcHandler::endElement( const QString &, const QString &,
				const QString &qName )
{
  if( qName == "name" )
    currentString += ":";
  else {
    /*if( (qName == "string") || (qName == "int") || (qName == "boolean")
      || (qName == "dateTime.iso8601" ) ) */
    if( qName == "value" )
      currentString += ";";
    else {
      if( qName == "struct" )
	{
	returnData.append( currentString );
	}
      else {
	if( qName == "methodResponse" )
	  methodResponseFinished = true;
      }
    }
  }
  return true;
}

bool XmlRpcHandler::fatalError( const QXmlParseException &exc )
{
  qWarning( "Line %d, column %d: %s", exc.lineNumber(),
	    exc.columnNumber(),
	    exc.message().toAscii().data() );
  returnData.append( QString( "Line %1, column %2: %3" )
		     .arg( exc.lineNumber() )
		     .arg( exc.columnNumber() )
		     .arg( exc.message() ) );
  methodResponseFinished = true;
  return false;
}

QList<QString> XmlRpcHandler::returnList()
{
  return returnData;
}

bool XmlRpcHandler::isMethodResponseFinished()
{
  return methodResponseFinished;
}
