/*******************************************************************************
 
    QTM - Qt-based blog manager
    Copyright (C) 2006 Matthew J Smith

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License (version 2), as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/

// catkin.h - Headers for QTM main window class.

#ifndef CATKIN_H
#define CATKIN_H

#include "ui_CatkinMainWindow.h"
#include "ui_SideWidget.h"
#include "ui_aboutbox.h"
//#include "ui_account_form.h"
#include "ui_ImageEntry.h"
#include "ui_LinkEntry.h"
#include "ui_password-form.h"
#include "PrefsDialog.h"
// #include "ui_ListDialog.h"
#ifdef USE_SYSTRAYICON
#include "SysTrayIcon.h"
#endif

class QWidget;
class QStackedWidget;
// class QSplitter;
class QTextBrowser;
class QPushButton;
class QAction;
class QResizeEvent;
class QComboBox;
//class QHttp;
class SafeHttp;
class QHttpResponseHeader;
class QTextEdit;
class QHBoxLayout;
class QRegExpValidator;
class QDomElement;
class QDomDocument;
class QHostInfo;
class QSystemTrayIcon;
class QMenu;

using namespace Ui;

class Catkin : public QMainWindow
{
  Q_OBJECT

 public:
  Catkin( bool noRefreshBlogs = false, QWidget *widget = 0 );
  Catkin( QList<QString>, QList<QString>, int,
	  QWidget *parent = 0 );
  Catkin( QString, QWidget *parent = 0 );
  ~Catkin();
#if QT_VERSION >= 0x040200
#ifdef USE_SYSTRAYICON
  void setSTI( SysTrayIcon * );
#endif
#endif

 private:
  Ui::MainWindow ui;
  Ui::CategoryWidget cw;
  void doUiSetup();
  void readSettings();
  void writeSettings();
  //void blogger_newPost( QByteArray );
  //void blogger_editPost( QByteArray );
  void blogger_getUsersBlogs( QByteArray );
  void metaWeblog_newPost( QByteArray );
  void metaWeblog_editPost( QByteArray );
  void mt_publishPost( QByteArray );
  void mt_getCategoryList( QByteArray );
  void mt_setPostCategories( QByteArray );
  //void submitBloggerEdit();
  void submitMTEdit();
  bool saveCheck();
  bool saveCheck( bool );
  QDomElement XmlValue( QDomDocument &, QString, QString,
			QString memberType = "param" );
  QDomElement XmlMember( QDomDocument &, QString, QString, QString );
  QDomElement XmlMethodName( QDomDocument &, QString );
  QDomElement XmlRpcArray( QDomDocument &, QString, QList<QString> );
  void setNetworkActionsEnabled( bool );
  //void newBloggerPost();
  void newMTPost(); // formerly blogThis()
  QString & getHTMLList( QString, QString & );

  int blogType, currentBlog, loadedEntryBlog;
  QString server;
  QString location;
  QString login;
  QString password;
  int port;
  QString localStorageDirectory;
  QString localStorageFileExtn;
  bool categoriesEnabled, entryBlogged, useNewWindows, savePassword,
    postAsSave, noPassword, initialChangeBlog, allowComments, allowTB, postDateTime,
    isBloggerBlog, bloggerHeaderFormatting;
  bool entryEverSaved, cleanSave;
  QList<QString> usersBlogs;
  QList<QString> categoryList;
  QByteArray responseData;
  QString entryNumber, dateTime;
  QString filename;
  int primaryCat;
  QString otherCatsList;
  QRegExpValidator *tagValidator;
  QString editorFontString, previewFontString, consoleFontString;
  bool useBloggerTitleFormatting;
  int bloggerTitleFormat;
  QStringList bloggerTitleFormatStrings;
  QLabel *dirtyIndicator;
  int STI2ClickFunction;
#ifdef USE_SYSTRAYICON
  SysTrayIcon *sti;
#endif

 private slots:
  void about();
  void choose();
  void save();
  void save( const QString & );
  void saveAs();
  void stopThisJob();
  void handleDone( bool );
  void handleResponseHeader( const QHttpResponseHeader & );
  void changeCurrentBlog( int );
  void changeBlog( int );
  void handleConsole( bool );
  void makeBold();
  void makeItalic();
  void makeUnderline();
  void makeBlockquote();
  void makePara();
  void insertLink();
  void insertLinkFromClipboard();
  void insertSelfLink();
  void insertImage();
  void insertImageFromClipboard();
  void insertMore();
  void cut();
  void copy();
  void paste();
  void pasteAsMarkedParagraphs();
  void pasteAsBlockquote();
  void pasteAsMarkdownBlockquote();
  void pasteAsUnorderedList();
  void pasteAsOrderedList();
  void undo();
  void redo();
  void makeUnorderedList();
  void makeOrderedList();
  void doPreview( bool );
  void showHighlightedURL( const QString & );
  void blogThis();
  void updatePostCategories();
  void setPostCategories();
  void publishPost();
  void setLoadedPostCategories();
  void doWhatsThis();
  void doViewBasicSettings();
  void doViewCategories();
  void doViewExcerpt();
  void doViewTechTags();
  void doViewTBPings();
  void addTechTag();
  void addClipTag();
  void addTechTagFromLineEdit();
  void addTechTagFromAddButton();
  void removeTechTag();
  void addTBPing();
  void addClipTBPing();
  void addTBPingFromLineEdit();
  void addTBPingFromAddButton();
  void removeTBPing();
  void handleInitialLookup( const QHostInfo & );
  void handleHostLookupFailed();
  void doFont();
  void doPreviewFont();
  void doConsoleFont();
  void configureQuickpostTemplates();

 public slots:
  void refreshCategories();
  void getPreferences();
  void refreshBlogList();
  void changeCaptionAfterTitleChanged();
  bool load( const QString &, QList<QString>, QList<QString> );
  bool load( const QString &, bool fromSTI = false );
  void dirtify();
  void doInitialChangeBlog();
  void newDoc();
  void doQuit();
  void setPublishStatus( int );

 protected:
  QStackedWidget *mainStack;
  QTextBrowser *previewWindow;
  QWidget *leftWidget, *cWidget;
  QPushButton *previewButton;
  QPushButton *blogThisButton;
  QAction *blogThisIntlAction;
  SafeHttp *http;
  QTextEdit *console;
  QTextEdit *ed;
  QHBoxLayout *mainWindowLayout;

  int previewWindowID;
  int consoleID, edID;
  int previousRaisedLSWidget;
  int currentHttpBusiness;
  // virtual void resizeEvent( QResizeEvent *e );
  virtual void closeEvent( QCloseEvent * );
  
signals:
  void httpBusinessFinished();
  void categoryRefreshFinished();
  //void loadingDone( QWidget *, bool );
};

#endif
